#!/usr/bin/wish4.0
# Program: provider
# Tcl version: 7.3 (Tcl/Tk/XF)
# Tk version: 3.6
# XF version: 2.3
#

# module inclusion
global env
global provList provColor
global cityX cityY cityProviders
global pOffsetX pOffsetY textSize

global argc
global argv
global tkVersion
global fontSize
global rectSide rectSpace
global zoomFactor lastZoom
global drawArrow 

set tmpArgv ""
set fontSize 50
set rectSide 4
set rectSpace 1
set drawArrow 0
set lastZoom 1
# standard binding for a listbox

  
proc set_button_for_list {list button} {
      set count [$list curselection]
#      puts "Current selection contains $count elements"
      if { [llength $count ] > 0 } {  \
	    $button  config -state normal 
	  } else { \
	    $button config -state disabled
      }

  }

proc activate_button_if { b args } {
  set count [llength $args ]
  set e [lindex $args 0]
  set v [$e get]
  set state [expr [string length $v] > 0 ]
  for { set i 0 } {$i <  $count} { incr i } {
       set e [lindex $args $i]
       set state [ expr $state && [string length [$e get]] > 0 ]
  }
  if { $state != 0 } {
      $b config -state normal
  } else {
      $b config -state disabled
  }
}

proc set_zoom { factor } {
    global lastZoom cities textSize
    if { $factor == $lastZoom } { return }
    set c .f.canvas
    set relative [ expr $factor / $lastZoom]
    set scregion [ $c cget -scrollregion ]
    set newregion {}
    foreach i $scregion {
	lappend newregion [expr $i * $relative]
    }
    $c config -scrollregion $newregion
    $c scale all 0.0 0.0 $relative $relative
    foreach i $cities {
	$c itemconfigure $i \
	    -font [make_font [expr round($textSize($i) * $factor )]]
	set tag ${i}_line
	if { [ $c gettags $tag ] != "" } {
	    set as [ $c itemcget $tag -arrowshape ]
	    set nas {} 
	    foreach s $as {
		lappend nas [expr $s * $relative]
	    }
	    $c itemconfigure $tag -arrowshape $nas
	}
    }			     
    set lastZoom $factor
}

proc save_canvas {} {
  set_zoom 1.0
  set content [ dump_canvas .f.canvas ]
  set f [open "karte.out" w]
  puts $f $content
  close $f
}

proc show_city {name } {
global cityX cityY
    global cityState provList fontSize cityProviders
    global textXoffset textYoffset zoomFactor textSize
    global drawArrow
#    puts "City exists: [info commands .city] name $name"
    if { [llength [info commands .city] ] > 0 } {
	wm deiconify .city 
    } else { 
	ShowWindow.city
    }
    .city.workarea.coords.xcoordinate.entry config -state normal
    .city.workarea.coords.ycoordinate.entry config -state normal
    .city.workarea.city.entry config -state normal
    .city.workarea.coords.xcoordinate.entry delete 0 end
    .city.workarea.coords.xcoordinate.entry insert 0 $cityX($name)
    .city.workarea.coords.ycoordinate.entry delete 0 end
    .city.workarea.coords.ycoordinate.entry insert 0 $cityY($name)
    .city.workarea.city.entry delete 0 end
    regsub -all "_" $name " " norm_name
    .city.workarea.city.entry insert 0 "$norm_name"
    .city.workarea.coords.xcoordinate.entry config -state disabled
    .city.workarea.coords.ycoordinate.entry config -state disabled
    .city.workarea.city.entry config -state disabled
    .city.actionarea.ok config -state normal
    set city $name
#
#   get ALL tags for this object
#
#    set taglist [lsort [.f.canvas gettags $city]]
#    puts "Tags for '$city': $taglist"
#
#   Add all providers from the tag list for this city to
#   the "selected providers" list
#
    .city.workarea.prov.frame11.sel_providers delete 0 end
    .city.workarea.prov.frame.all_providers delete 0 end
    set available [lsort $cityProviders($city)]
    foreach i $available {
	.city.workarea.prov.frame11.sel_providers \
	    insert end $i
    }

    foreach i $provList {
	if { [ lsearch $cityProviders($city) $i] < 0 } {
	    .city.workarea.prov.frame.all_providers insert end $i
	}
    }
    
    if { [.f.canvas gettags ${city}_line] == "" } {
	set drawArrow 0
    } else {
	set drawArrow 1
    }
    set currentTextPos [.f.canvas coords $city]
    set textXoffset \
    [ expr ([lindex $currentTextPos 0] / $zoomFactor) - $cityX($city) ]
    set textYoffset \
    [ expr ([lindex $currentTextPos 1] / $zoomFactor) - $cityY($city)]
    if { [info exists textSize($city) ] } {
	set fontSize $textSize($city)
    } else {
	set fs [get_font_size [.f.canvas itemcget $city -font]]
        puts "FontSize for $city: $fs"
	set fontSize [expr round($fs / $zoomFactor)]
	set textSize($city) $fontSize
    }
    set cityState "change"
}


proc terminate {} {
  save_canvas
  destroy .
}

# NB. Requires Tcl7.4 (and probably Tk4.0) though it's easy to (part)
#   lobotomize, and it does not support restoration of the focus or
#   indices. Restoring to a widget in a different context is probably
#   unwise, especially if your programming is as bad as mine... :^)
proc dump_canvas {c} {
    global rectSide rectSpace pOffsetX pOffsetY cityX cityY cities textSize
    # c     - canvas ident
    set retstr ""
    foreach tg {current all} {
	# Get the bindings for predefined tags
	set donetag($tg) 1
	set bind ""
	foreach sq [$c bind $tg] {
	    lappend bind [list $sq [$c bind $tg $sq]]
	}
	# append to output
	append retstr [list [list {} $tg {} $bind]] "\n"
    }
    foreach oid [$c find all] {
	# get the state
	set type    [$c type $oid]
	set coord   [$c coords $oid]
#	puts "oid $oid, type: $type"
	# get the item bindings
	set donetag($oid) 1
	set bind    ""
	foreach sq [$c bind $oid] {
	    lappend bind [list $sq [$c bind $oid $sq]]
	}
	# get the item configuration
	set cfg     ""
	foreach cf [$c itemconfigure $oid] {
	    lappend cfg [list [lindex $cf 0] [lindex $cf end]]
	}
	# append to output
	append retstr [list [list $type $coord $cfg $bind]] "\n"
	# add any tag bindings not already dealt with
	foreach tg [$c gettags $oid] {
	    if [catch {set donetag($tg)}] {
		set donetag($tg) 1
		set bind ""
		foreach sq [$c bind $tg] {
		    lappend bind [list $sq [$c bind $tg $sq]]
		}
		append retstr [list [list {} $tg {} $bind]] "\n"
	    }
	}
    }
    # You'd probably want to add something to save the state of canvas 
    # itself, as opposed to the items therein.
    append retstr "global rectSide rectSpace pOffsetX pOffsetY cityX cityY\n"
    append retstr "global textSize"
    append retstr "set rectSide $rectSide \n"
    append retstr "set rectSpace $rectSpace\n"
    foreach c $cities {
	if { [info exists pOffsetX($c) ] } {
	    append retstr \
	    "set pOffsetX($c) $pOffsetX($c) \nset pOffsetY($c) $pOffsetY($c)\n"
	}

	if { [info exists textSize($c)]} {
	append retstr \
	    "set textSize($c) $textSize($c)\n"
	}
	    
	append retstr \
	    "set cityX($c) $cityX($c)\nset cityY($c) $cityY($c)\n"
    }
    return $retstr
}

proc undump_canvas {c state} {
    # c      - canvas ident
    # state  - state generated with dump_canvas above
    foreach i $state {
	if [catch {
	    # Make the object
	    set id [eval $c create [lindex $i 0] [lindex $i 1]]
#	    catch {
		# configure it
		foreach cf [lindex $i 2] {
		    eval $c itemconfigure $id $cf
#		}
	    }
	}] {
	    # Horrible hack for non-item bindings
	    set id [lindex $i 1]
	}
	# bind it
	foreach bd [lindex $i 3] {
	    eval $c bind $id $bd
	}
    }
}

proc print_canvas {} {
    set_zoom 1.0
    set corners [.f.canvas cget -scrollregion]
    set x [lindex $corners 0]
    set y [lindex $corners 1]
    set w [ expr [lindex $corners 2] - $x ]
    set h [ expr [lindex $corners 3] - $y ]
    .f.canvas itemconfigure border -fill ""
    set ps_string [.f.canvas postscript \
		       -colormode color -height $h -width $w -x $x -y $y \
		       -pageanchor sw -pagex 0 -pagey $y ]
    regsub -all "showpage" $ps_string "" new_string
    regsub {[0-9.]+ +[0-9.]+ +scale} $new_string "" ps_string
    set in [open "border.eps" "r"]
    set out [open "karte.eps" "w"]
    puts $out $ps_string
    while { [gets $in line] >= 0 } {
	puts $out $line
    }
    close $in
    close $out
    .f.canvas itemconfigure border -fill black
}

# procedure to show window .city
proc ShowWindow.city {args} {# xf ignore me 7
    global fontSize
  # build widget .city
  if {"[info procs XFEdit]" != ""} {
    catch "XFDestroy .city"
  } {
    catch "destroy .city"
  }
  toplevel .city 

  # Window manager configurations
  global tkVersion
  wm positionfrom .city ""
  wm sizefrom .city user
  wm maxsize .city 1000 768
  wm minsize .city 10 10
  wm title .city {city}


  # build widget .city.actionarea
  frame .city.actionarea

  # build widget .city.actionarea.ok
  button .city.actionarea.ok \
    -command { terminate_city
               wm withdraw .city} \
    -state disabled  \
    -text {OK}

  # build widget .city.actionarea.button8
  button .city.actionarea.button8 \
      -command {terminate_city} \
      -text {Anwenden}

  # build widget .city.actionarea.button9
  button .city.actionarea.button9 \
      -command { wm withdraw .city } \
      -text {Abbrechen}

  # pack widget .city.actionarea
  pack append .city.actionarea \
    .city.actionarea.ok {left frame nw padx 3 expand fillx} \
    .city.actionarea.button8 {left frame center padx 3 expand fillx} \
    .city.actionarea.button9 {left frame se padx 3 expand fillx}

  # build widget .city.workarea
  frame .city.workarea

  # build widget .city.workarea.city
  frame .city.workarea.city

  # build widget .city.workarea.city.label
  label .city.workarea.city.label \
    -borderwidth {0} \
    -padx {2} \
    -relief {raised} \
    -text {Ort:} \
    -width {6}

  # build widget .city.workarea.city.entry
  entry .city.workarea.city.entry \
    -relief {groove} \
    -width {25}
   
  bind .city.workarea.city.entry <KeyRelease> \
	  "activate_button_if .city.actionarea.ok \
	   .city.workarea.city.entry \
	   .city.workarea.coords.xcoordinate.entry \
	   .city.workarea.coords.ycoordinate.entry "


  # pack widget .city.workarea.city
  pack append .city.workarea.city \
    .city.workarea.city.label {left frame center} \
    .city.workarea.city.entry {top frame center expand fill}

  # build widget .city.workarea.coords
  frame .city.workarea.coords

  # build widget .city.workarea.coords.xcoordinate
  frame .city.workarea.coords.xcoordinate

  # build widget .city.workarea.coords.xcoordinate.label
  label .city.workarea.coords.xcoordinate.label \
    -padx {2} \
    -text {X:}

  # build widget .city.workarea.coords.xcoordinate.entry
  entry .city.workarea.coords.xcoordinate.entry \
    -relief {groove}

  bind .city.workarea.coords.xcoordinate.entry <KeyRelease> \
	  "activate_button_if .city.actionarea.ok \
	   .city.workarea.city.entry \
	   .city.workarea.coords.xcoordinate.entry \
	   .city.workarea.coords.ycoordinate.entry "

  # pack widget .city.workarea.coords.xcoordinate
  pack append .city.workarea.coords.xcoordinate \
    .city.workarea.coords.xcoordinate.label {left frame center padx 26} \
    .city.workarea.coords.xcoordinate.entry {top frame center expand fill}

  # build widget .city.workarea.coords.ycoordinate
  frame .city.workarea.coords.ycoordinate

  # build widget .city.workarea.coords.ycoordinate.label
  label .city.workarea.coords.ycoordinate.label \
    -padx {2} \
    -text {Y:}

  # build widget .city.workarea.coords.ycoordinate.entry
  entry .city.workarea.coords.ycoordinate.entry \
    -relief {groove}

  bind .city.workarea.coords.ycoordinate.entry <KeyRelease> \
	  "activate_button_if .city.actionarea.ok \
	   .city.workarea.city.entry \
	   .city.workarea.coords.xcoordinate.entry \
	   .city.workarea.coords.ycoordinate.entry "

  # pack widget .city.workarea.coords.ycoordinate
  pack append .city.workarea.coords.ycoordinate \
    .city.workarea.coords.ycoordinate.label {left frame center} \
    .city.workarea.coords.ycoordinate.entry {top frame center expand fill}

  # pack widget .city.workarea.coords
  pack append .city.workarea.coords \
    .city.workarea.coords.xcoordinate {left frame center padx 18 fill} \
    .city.workarea.coords.ycoordinate {top frame center fill}

  frame .city.workarea.sizes -relief flat

  # build widget .city.workarea.fontframe
  frame .city.workarea.sizes.fontframe -relief groove -borderwidth 3
  radiobutton .city.workarea.sizes.fontframe.p05 \
      -text "5 Punkt" -variable fontSize -value 50 -anchor w \
	-command {scale_text $fontSize}
  radiobutton .city.workarea.sizes.fontframe.p06 \
      -text "6 Punkt" -variable fontSize -value 60 -anchor w \
	-command {scale_text $fontSize}
  radiobutton .city.workarea.sizes.fontframe.p07 \
      -text "7 Punkt" -variable fontSize -value 70 -anchor w \
	-command {scale_text $fontSize}
    

  pack \
      .city.workarea.sizes.fontframe.p05 \
      .city.workarea.sizes.fontframe.p06 \
      .city.workarea.sizes.fontframe.p07 \
      -side top -fill x  

  # build widget city.workarea.moveframe
  frame .city.workarea.sizes.moveframe -relief groove -borderwidth 3
  scale .city.workarea.sizes.moveframe.movex \
	-from -50.0 -to 50.0 -showvalue 1 -tickinterval 0 \
	-variable textXoffset -orient horizontal \
	-command move_text
  scale .city.workarea.sizes.moveframe.movey \
	-from -50.0 -to 50.0 -showvalue 1 -tickinterval 0 \
	-variable textYoffset -orient vertical \
	-command move_text

  frame .city.workarea.sizes.moveframe.rbox -relief groove -borderwidth 3

  radiobutton .city.workarea.sizes.moveframe.rbox.moveText \
	-text "Text verschieben" -anchor w \
	-variable moveWhat -value "text" \
	-command {
	    set l [get_text_offset [.city.workarea.city.entry get]]
	    set textXoffset [lindex $l 0]
	    set textYoffset [lindex $l 1]
	}
  radiobutton .city.workarea.sizes.moveframe.rbox.moveProvider \
	-text "Provider verschieben" -anchor w \
	-variable moveWhat -value "provider" \
	-command { 
	    set l [get_provider_offset [.city.workarea.city.entry get]]
	    set textXoffset [lindex $l 0]
	    set textYoffset [lindex $l 1]
	}
  checkbutton .city.workarea.sizes.moveframe.arrowbutton \
	-text "Pfeil zeichnen" -command {draw_arrow} -variable drawArrow

  .city.workarea.sizes.moveframe.rbox.moveProvider deselect
  .city.workarea.sizes.moveframe.rbox.moveText select

  pack .city.workarea.sizes.moveframe.rbox.moveText \
	.city.workarea.sizes.moveframe.rbox.moveProvider \
	-side top -anchor w

  pack 	.city.workarea.sizes.moveframe.movey \
	-side left  -fill x
  pack	.city.workarea.sizes.moveframe.rbox \
	-side top
  pack .city.workarea.sizes.moveframe.arrowbutton \
	-side bottom -anchor w
  pack	.city.workarea.sizes.moveframe.movex \
    -side bottom -anchor sw -fill x


  pack append .city.workarea.sizes \
	.city.workarea.sizes.fontframe { left frame center expand fill } \
	.city.workarea.sizes.moveframe { left frame center expand fill } 


  # build widget .city.workarea.prov
  frame .city.workarea.prov

  # build widget .city.workarea.prov.frame
  frame .city.workarea.prov.frame \
    -borderwidth {3}

  # build widget .city.workarea.prov.frame.scrollbar3
  scrollbar .city.workarea.prov.frame.scrollbar3 \
    -command {.city.workarea.prov.frame.all_providers xview} \
    -orient {horizontal} \
    -relief {raised}

  # build widget .city.workarea.prov.frame.scrollbar2
  scrollbar .city.workarea.prov.frame.scrollbar2 \
    -command {.city.workarea.prov.frame.all_providers yview} \
    -relief {raised}

  # build widget .city.workarea.prov.frame.all_providers
  listbox .city.workarea.prov.frame.all_providers \
    -relief {raised} \
    -xscrollcommand {.city.workarea.prov.frame.scrollbar3 set} \
    -yscrollcommand {.city.workarea.prov.frame.scrollbar2 set} 


  # pack widget .city.workarea.prov.frame
  pack append .city.workarea.prov.frame \
    .city.workarea.prov.frame.scrollbar2 {right frame center filly} \
    .city.workarea.prov.frame.all_providers {top frame center expand fill} \
    .city.workarea.prov.frame.scrollbar3 {bottom frame center fillx}

  # build widget .city.workarea.prov.frame11
  frame .city.workarea.prov.frame11 \
    -borderwidth {3}

  # build widget .city.workarea.prov.frame11.scrollbar3
  scrollbar .city.workarea.prov.frame11.scrollbar3 \
    -command {.city.workarea.prov.frame11.sel_providers xview} \
    -orient {horizontal} \
    -relief {raised}

  # build widget .city.workarea.prov.frame11.scrollbar2
  scrollbar .city.workarea.prov.frame11.scrollbar2 \
    -command {.city.workarea.prov.frame11.sel_providers yview} \
    -relief {raised}

  # build widget .city.workarea.prov.frame11.sel_providers
  listbox .city.workarea.prov.frame11.sel_providers \
    -relief {raised} \
    -xscrollcommand {.city.workarea.prov.frame11.scrollbar3 set} \
    -yscrollcommand {.city.workarea.prov.frame11.scrollbar2 set}

  # pack widget .city.workarea.prov.frame11
  pack append .city.workarea.prov.frame11 \
    .city.workarea.prov.frame11.scrollbar2 {right frame center filly} \
    .city.workarea.prov.frame11.sel_providers {top frame center expand fill} \
    .city.workarea.prov.frame11.scrollbar3 {bottom frame center fillx}

  # build widget .city.workarea.prov.frame2
  frame .city.workarea.prov.frame2

  # build widget .city.workarea.prov.frame2.button0
  button .city.workarea.prov.frame2.button0 \
    -state {disabled} \
    -text {Hinzufgen} \
    -command  { \
    set toadd [.city.workarea.prov.frame.all_providers curselection] 
    set existing [.city.workarea.prov.frame11.sel_providers get 0 end] 
      foreach i $toadd { 
        set item [.city.workarea.prov.frame.all_providers get $i]
        if { [lsearch $existing $item] < 0 } { 
        .city.workarea.prov.frame11.sel_providers insert end $item } 
	.city.workarea.prov.frame.all_providers delete $i
    }
#   .city.workarea.prov.frame.all_providers selection clear 0 end
    .city.workarea.prov.frame2.button0 config -state disabled
  }   
  # build widget .city.workarea.prov.frame2.button1
  button .city.workarea.prov.frame2.button1 \
    -state {disabled} \
    -text {Lschen} \
    -command { \
       set remove [.city.workarea.prov.frame11.sel_providers curselection] 
       foreach i $remove { 
	 .city.workarea.prov.frame.all_providers insert end \
	     [ .city.workarea.prov.frame11.sel_providers get $i]
         .city.workarea.prov.frame11.sel_providers delete $i 
       }
       .city.workarea.prov.frame11.sel_providers selection clear 0 end
       .city.workarea.prov.frame2.button1 config -state disabled
     }   
  # pack widget .city.workarea.prov.frame2
  pack append .city.workarea.prov.frame2 \
    .city.workarea.prov.frame2.button0 {top frame center expand fillx} \
    .city.workarea.prov.frame2.button1 {top frame center pady 19 expand fillx}

  # build widget .city.workarea.prov.frame3
  frame .city.workarea.prov.frame3

  # build widget .city.workarea.prov.frame3.label4
  label .city.workarea.prov.frame3.label4 \
    -text {Vorhandene Provider}

  # pack widget .city.workarea.prov.frame3
  pack append .city.workarea.prov.frame3 \
    .city.workarea.prov.frame3.label4 {right frame ne padx 33 fillx}

  # pack widget .city.workarea.prov
  pack append .city.workarea.prov \
    .city.workarea.prov.frame3 {top frame center fill} \
    .city.workarea.prov.frame {left frame center fill} \
    .city.workarea.prov.frame2 {left frame center fill} \
    .city.workarea.prov.frame11 {top frame center fill}

  # pack widget .city.workarea
    pack append .city.workarea \
	.city.workarea.city {top frame center fill} \
	.city.workarea.coords {top frame center fill} \
	.city.workarea.sizes {top frame center fill } \
	.city.workarea.prov {top frame center fill}

  # pack widget .city
  pack append .city \
    .city.workarea {top frame n expand fill} \
    .city.actionarea {bottom frame center fillx}

  .city.workarea.city.entry insert end {}
  .city.workarea.coords.xcoordinate.entry insert end {}
  .city.workarea.coords.ycoordinate.entry insert end {}

#  puts [format "Bindings for list: %s" \
#          [bind .city.workarea.prov.frame.all_providers]]
  bind .city.workarea.prov.frame.all_providers <ButtonRelease-1> \
	  "+set_button_for_list .city.workarea.prov.frame.all_providers \
	  .city.workarea.prov.frame2.button0"
  
  bind .city.workarea.prov.frame11.sel_providers <ButtonRelease-1> \
	   "+set_button_for_list .city.workarea.prov.frame11.sel_providers \
	   .city.workarea.prov.frame2.button1 "
 


  if {"[info procs XFEdit]" != ""} {
    catch "XFMiscBindWidgetTree .city"
    after 2 "catch {XFEditSetShowWindows}"
  }
}

proc DestroyWindow.city {} {# xf ignore me 7
  if {"[info procs XFEdit]" != ""} {
    if {"[info commands .city]" != ""} {
      global xfShowWindow.city
      set xfShowWindow.city 0
      XFEditSetPath .
      after 2 "XFSaveAsProc .city; XFEditSetShowWindows"
    }
  } {
    catch "destroy .city"
    update
  }
}


# procedure to show window .
proc ShowWindow. {args} {# xf ignore me 7

  # Window manager configurations
  global tkVersion
  wm positionfrom . user
  wm sizefrom . ""
  wm maxsize . 1024 768
  wm title . {xf #2}


  # build widget menu
  frame .menu \
    -borderwidth {2} \
    -relief {raised}

  # build widget .menu.file
  menubutton .menu.file \
    -menu {.menu.file.m} \
    -text {Datei} \
    -underline {0}

  # build widget .menu.file.m
  menu .menu.file.m
  .menu.file.m add command \
    -label {Laden} \
    -underline {0}
  .menu.file.m add command \
    -label {Speichern} \
    -command { save_canvas } \
    -underline {0}
  .menu.file.m add command \
    -label {Speichern als} \
    -underline {10}
  .menu.file.m add separator
  .menu.file.m add command \
    -label {Drucken} \
    -command {print_canvas} \
    -underline {0}
  .menu.file.m add separator
  .menu.file.m add command \
    -accelerator {<Ctrl>-E} \
    -command { terminate } \
    -label {Ende} \
    -underline {0}

  # build widget .menu.help
  menubutton .menu.help \
    -menu {.menu.help.m} \
    -text {Help} \
    -underline {0}

  # build widget .menu.help.m
  menu .menu.help.m

  # build widget .menu.city
  menubutton .menu.city \
    -menu {.menu.city.m} \
    -text {Ort} \
    -underline {0}

  menubutton .menu.zoom \
    -menu {.menu.zoom.m} \
    -text {Ansicht} \
    -underline {0}


  # build widget .menu.city.m
  menu .menu.city.m
  .menu.city.m add command \
    -command { \
    if { [info exists .city] } {\
      wm deiconify .city } \
      else { \
	  global cityState \
	  set cityState "new" \
	  ShowWindow.city } \
    } \
    -label {Neu} \
    -underline {0}
      
      menu .menu.zoom.m
      .menu.zoom.m add radiobutton  \
		  -command "set_zoom 1.0" \
		  -label "100%" \
		  -underline 0 \
		  -variable zoomFactor \
		  -value 1.

      .menu.zoom.m add radiobutton  \
		  -command "set_zoom 2.0" \
		  -label "200%" \
		  -underline 0 \
		  -variable zoomFactor \
		  -value 2.

      .menu.zoom.m add radiobutton  \
		  -command "set_zoom 3.0" \
		  -label "300%" \
		  -underline 0 \
		  -variable zoomFactor \
		  -value 3.

      .menu.zoom.m add radiobutton  \
		  -command "set_zoom 4.0" \
		  -label "400%" \
		  -underline 0 \
		  -variable zoomFactor \
		  -value 4.

      .menu.zoom.m add radiobutton  \
		  -command "set_zoom 0.5" \
		  -label "50%" \
		  -underline 0 \
		  -variable zoomFactor \
		  -value 0.5

  # pack widget .menu
  pack append .menu \
    .menu.file {left frame center} \
    .menu.help {right frame center} \
    .menu.city {left frame center} \
    .menu.zoom {left frame center}

  frame .f -relief flat
  scrollbar .f.xscroll_canvas -command ".f.canvas xview" -orient horizontal
  scrollbar .f.yscroll_canvas -command ".f.canvas yview" -orient vertical

  # build widget .f.canvas
  canvas .f.canvas \
	  -scrollregion {0 0 597 842} \
    -height {500} \
    -insertofftime {600} \
    -relief {raised} \
    -yscrollcommand ".f.yscroll_canvas set" \
    -xscrollcommand ".f.xscroll_canvas set" \
    -width {600} \
    -background white

  # pack widget .f
  pack append .f \
  .f.yscroll_canvas {right frame center filly} \
  .f.canvas {top frame center expand fill} \
  .f.xscroll_canvas {top frame center fillx}

  # pack widget .
  pack append . \
    .menu {top frame center fill} \
    .f {top frame center expand fill}

  global tkVersion tk_version
  if { ! [info exists tkVersion]} { set tkVersion $tk_version }

  if {$tkVersion >= 3.0} {
    tk_menuBar .menu .menu.file .menu.help .menu.city
  } {
    tk_menus . .menu.file .menu.help .menu.city
  }

  # build canvas items .canvas
#  set xfTmpTag [.canvas create polygon 0 0 0 500 500 500 500 0 0 0]
#  .canvas itemconfigure $xfTmpTag \
#    -fill {white}



  if {"[info procs XFEdit]" != ""} {
    catch "XFMiscBindWidgetTree ."
    after 2 "catch {XFEditSetShowWindows}"
  }
}


# User defined procedures


# Procedure: FSBox
proc FSBox { {fsBoxMessage "Select file:"} {fsBoxFileName ""} {fsBoxActionOk ""} {fsBoxActionCancel ""}} {
# xf ignore me 5
##########
# Procedure: FSBox
# Description: show file selector box
# Arguments: fsBoxMessage - the text to display
#            fsBoxFileName - a file name that should be selected
#            fsBoxActionOk - the action that should be performed on ok
#            fsBoxActionCancel - the action that should be performed on cancel
# Returns: the filename that was selected, or nothing
# Sideeffects: none
##########
# 
# global fsBox(activeBackground) - active background color
# global fsBox(activeForeground) - active foreground color
# global fsBox(background) - background color
# global fsBox(font) - text font
# global fsBox(foreground) - foreground color
# global fsBox(extensions) - scan directory for extensions
# global fsBox(scrollActiveForeground) - scrollbar active background color
# global fsBox(scrollBackground) - scrollbar background color
# global fsBox(scrollForeground) - scrollbar foreground color
# global fsBox(scrollSide) - side where scrollbar is located

  global fsBox

  set tmpButtonOpt ""
  set tmpFrameOpt ""
  set tmpMessageOpt ""
  set tmpScaleOpt ""
  set tmpScrollOpt ""
  if {"$fsBox(activeBackground)" != ""} {
    append tmpButtonOpt "-activebackground \"$fsBox(activeBackground)\" "
  }
  if {"$fsBox(activeForeground)" != ""} {
    append tmpButtonOpt "-activeforeground \"$fsBox(activeForeground)\" "
  }
  if {"$fsBox(background)" != ""} {
    append tmpButtonOpt "-background \"$fsBox(background)\" "
    append tmpFrameOpt "-background \"$fsBox(background)\" "
    append tmpMessageOpt "-background \"$fsBox(background)\" "
  }
  if {"$fsBox(font)" != ""} {
    append tmpButtonOpt "-font \"$fsBox(font)\" "
    append tmpMessageOpt "-font \"$fsBox(font)\" "
  }
  if {"$fsBox(foreground)" != ""} {
    append tmpButtonOpt "-foreground \"$fsBox(foreground)\" "
    append tmpMessageOpt "-foreground \"$fsBox(foreground)\" "
  }
  if {"$fsBox(scrollActiveForeground)" != ""} {
    append tmpScrollOpt "-activeforeground \"$fsBox(scrollActiveForeground)\" "
  }
  if {"$fsBox(scrollBackground)" != ""} {
    append tmpScrollOpt "-background \"$fsBox(scrollBackground)\" "
  }
  if {"$fsBox(scrollForeground)" != ""} {
    append tmpScrollOpt "-foreground \"$fsBox(scrollForeground)\" "
  }

  if {[file exists [file tail $fsBoxFileName]] &&
      [IsAFile [file tail $fsBoxFileName]]} {
    set fsBox(name) [file tail $fsBoxFileName]
  } {
    set fsBox(name) ""
  }
  if {[file exists $fsBoxFileName] && [IsADir $fsBoxFileName]} {
    set fsBox(path) $fsBoxFileName
  } {
    if {"[file dirname $fsBoxFileName]" != "."} {
      set fsBox(path) [file dirname $fsBoxFileName]
    }
  }
  if {$fsBox(showPixmap)} {
    set fsBox(path) [string trimleft $fsBox(path) @]
  }
  if {"$fsBox(path)" != "" && [file exists $fsBox(path)] &&
      [IsADir $fsBox(path)]} {
    set fsBox(internalPath) $fsBox(path)
  } {
    if {"$fsBox(internalPath)" == "" ||
        ![file exists $fsBox(internalPath)]} {
      set fsBox(internalPath) [pwd]
    }
  }
  # build widget structure

  # start build of toplevel
  if {"[info commands XFDestroy]" != ""} {
    catch {XFDestroy .fsBox}
  } {
    catch {destroy .fsBox}
  }
  toplevel .fsBox  -borderwidth 0
  catch ".fsBox config $tmpFrameOpt"
  wm geometry .fsBox 350x300 
  wm title .fsBox {File select box}
  wm maxsize .fsBox 1000 1000
  wm minsize .fsBox 100 100
  # end build of toplevel

  label .fsBox.message1  -anchor c  -relief raised  -text "$fsBoxMessage"
  catch ".fsBox.message1 config $tmpMessageOpt"

  frame .fsBox.frame1  -borderwidth 0  -relief raised
  catch ".fsBox.frame1 config $tmpFrameOpt"

  button .fsBox.frame1.ok  -text "OK"  -command "
      global fsBox
      set fsBox(name) \[.fsBox.file.file get\]
      if {$fsBox(showPixmap)} {
        set fsBox(path) @\[.fsBox.path.path get\]
      } {
        set fsBox(path) \[.fsBox.path.path get\]
      }
      set fsBox(internalPath) \[.fsBox.path.path get\]
      $fsBoxActionOk
      if {\"\[info commands XFDestroy\]\" != \"\"} {
        catch {XFDestroy .fsBox}
      } {
        catch {destroy .fsBox}
      }"
  catch ".fsBox.frame1.ok config $tmpButtonOpt"

  button .fsBox.frame1.rescan  -text "Rescan"  -command {
      global fsBox
      FSBoxFSShow [.fsBox.path.path get]  [.fsBox.pattern.pattern get] $fsBox(all)}
  catch ".fsBox.frame1.rescan config $tmpButtonOpt"

  button .fsBox.frame1.cancel  -text "Cancel"  -command "
      global fsBox
      set fsBox(name) {}
      set fsBox(path) {}
      $fsBoxActionCancel
      if {\"\[info commands XFDestroy\]\" != \"\"} {
        catch {XFDestroy .fsBox}
      } {
        catch {destroy .fsBox}
      }"
  catch ".fsBox.frame1.cancel config $tmpButtonOpt"

  if {$fsBox(showPixmap)} {
    frame .fsBox.frame2  -borderwidth 0  -relief raised
    catch ".fsBox.frame2 config $tmpFrameOpt"

    scrollbar .fsBox.frame2.scrollbar3  -command {.fsBox.frame2.canvas2 xview}  -orient {horizontal}  -relief {raised}
    catch ".fsBox.frame2.scrollbar3 config $tmpScrollOpt"

    scrollbar .fsBox.frame2.scrollbar1  -command {.fsBox.frame2.canvas2 yview}  -relief {raised}
    catch ".fsBox.frame2.scrollbar1 config $tmpScrollOpt"

    canvas .fsBox.frame2.canvas2  -confine {true}  -relief {raised}  -scrollregion {0c 0c 20c 20c}  -width {100}  -xscrollcommand {.fsBox.frame2.scrollbar3 set}  -yscrollcommand {.fsBox.frame2.scrollbar1 set}
    catch ".fsBox.frame2.canvas2 config $tmpFrameOpt"

    .fsBox.frame2.canvas2 addtag currentBitmap withtag [.fsBox.frame2.canvas2 create bitmap 5 5 -anchor nw]
  }

  frame .fsBox.path  -borderwidth 0  -relief raised
  catch ".fsBox.path config $tmpFrameOpt"

  frame .fsBox.path.paths  -borderwidth 2  -relief raised
  catch ".fsBox.path.paths config $tmpFrameOpt"

  menubutton .fsBox.path.paths.paths  -borderwidth 0  -menu ".fsBox.path.paths.paths.menu"  -relief flat  -text "Pathname:"
  catch ".fsBox.path.paths.paths config $tmpButtonOpt"

  menu .fsBox.path.paths.paths.menu
  catch ".fsBox.path.paths.paths.menu config $tmpButtonOpt"

  .fsBox.path.paths.paths.menu add command  -label "[string trimright $fsBox(internalPath) {/@}]"  -command "
       global fsBox
       FSBoxFSShow \[.fsBox.path.path get\]  \[.fsBox.pattern.pattern get\] \$fsBox(all)
       .fsBox.path.path delete 0 end
       .fsBox.path.path insert 0 [string trimright $fsBox(internalPath) {/@}]"

  entry .fsBox.path.path  -relief raised
  catch ".fsBox.path.path config $tmpMessageOpt"

  if {![IsADir $fsBox(internalPath)]} {
    set $fsBox(internalPath) [pwd]
  }
  .fsBox.path.path insert 0 $fsBox(internalPath)

  frame .fsBox.pattern  -borderwidth 0  -relief raised
  catch ".fsBox.pattern config $tmpFrameOpt"

  frame .fsBox.pattern.patterns  -borderwidth 2  -relief raised
  catch ".fsBox.pattern.patterns config $tmpFrameOpt"

  menubutton .fsBox.pattern.patterns.patterns  -borderwidth 0  -menu ".fsBox.pattern.patterns.patterns.menu"  -relief flat  -text "Selection pattern:"
  catch ".fsBox.pattern.patterns.patterns config $tmpButtonOpt"

  menu .fsBox.pattern.patterns.patterns.menu
  catch ".fsBox.pattern.patterns.patterns.menu config $tmpButtonOpt"

  .fsBox.pattern.patterns.patterns.menu add checkbutton  -label "Scan extensions"  -variable fsBox(extensions)  -command {
      global fsBox
      FSBoxFSShow [.fsBox.path.path get]  [.fsBox.pattern.pattern get] $fsBox(all)}

  entry .fsBox.pattern.pattern  -relief raised
  catch ".fsBox.pattern.pattern config $tmpMessageOpt"

  .fsBox.pattern.pattern insert 0 $fsBox(pattern)
  
  frame .fsBox.files  -borderwidth 0  -relief raised
  catch ".fsBox.files config $tmpFrameOpt"

  scrollbar .fsBox.files.vscroll  -relief raised  -command ".fsBox.files.files yview"
  catch ".fsBox.files.vscroll config $tmpScrollOpt"

  scrollbar .fsBox.files.hscroll  -orient horiz  -relief raised  -command ".fsBox.files.files xview"
  catch ".fsBox.files.hscroll config $tmpScrollOpt"

  listbox .fsBox.files.files  -exportselection false  -relief raised  -xscrollcommand ".fsBox.files.hscroll set"  -yscrollcommand ".fsBox.files.vscroll set"
  catch ".fsBox.files.files config $tmpMessageOpt"

  frame .fsBox.file  -borderwidth 0  -relief raised
  catch ".fsBox.file config $tmpFrameOpt"

  label .fsBox.file.labelfile  -relief raised  -text "Filename:"
  catch ".fsBox.file.labelfile config $tmpMessageOpt"

  entry .fsBox.file.file  -relief raised
  catch ".fsBox.file.file config $tmpMessageOpt"

  .fsBox.file.file delete 0 end
  .fsBox.file.file insert 0 $fsBox(name)
  
  checkbutton .fsBox.pattern.all  -offvalue 0  -onvalue 1  -text "Show all files"  -variable fsBox(all)  -command {
      global fsBox
      FSBoxFSShow [.fsBox.path.path get]  [.fsBox.pattern.pattern get] $fsBox(all)}
  catch ".fsBox.pattern.all config $tmpButtonOpt"

  FSBoxFSShow $fsBox(internalPath) $fsBox(pattern) $fsBox(all)

  # bindings
  bind .fsBox.files.files <Double-Button-1> "
    FSBoxFSFileSelectDouble %W $fsBox(showPixmap) \{$fsBoxActionOk\} %y"
  bind .fsBox.files.files <ButtonPress-1> "
    FSBoxFSFileSelect %W $fsBox(showPixmap) %y"
  bind .fsBox.files.files <Button1-Motion> "
    FSBoxFSFileSelect %W $fsBox(showPixmap) %y"
  bind .fsBox.files.files <Shift-Button1-Motion> "
    FSBoxFSFileSelect %W $fsBox(showPixmap) %y"
  bind .fsBox.files.files <Shift-ButtonPress-1> "
    FSBoxFSFileSelect %W $fsBox(showPixmap) %y"

  bind .fsBox.path.path <Tab> {
    FSBoxFSNameComplete path}
  bind .fsBox.path.path <Return> {
    global tkVersion
    global fsBox
    FSBoxFSShow [.fsBox.path.path get]  [.fsBox.pattern.pattern get] $fsBox(all)
    FSBoxFSInsertPath
    if {$tkVersion >= 3.0} {
      .fsBox.file.file icursor end
    } {
      .fsBox.file.file cursor end
    }
    focus .fsBox.file.file}
  catch "bind .fsBox.path.path <Up> {}"
  bind .fsBox.path.path <Down> {
    global tkVersion
    if {$tkVersion >= 3.0} {
      .fsBox.file.file icursor end
    } {
      .fsBox.file.file cursor end
    }
    focus .fsBox.file.file}

  bind .fsBox.file.file <Tab> {
    FSBoxFSNameComplete file}
  bind .fsBox.file.file <Return> "
    global fsBox
    set fsBox(name) \[.fsBox.file.file get\]
    if {$fsBox(showPixmap)} {
      set fsBox(path) @\[.fsBox.path.path get\]
    } {
      set fsBox(path) \[.fsBox.path.path get\]
    }
    set fsBox(internalPath) \[.fsBox.path.path get\]
    $fsBoxActionOk
    if {\"\[info commands XFDestroy\]\" != \"\"} {
      catch {XFDestroy .fsBox}
    } {
      catch {destroy .fsBox}
    }"
  bind .fsBox.file.file <Up> {
    global tkVersion
    if {$tkVersion >= 3.0} {
      .fsBox.path.path icursor end
    } {
      .fsBox.path.path cursor end
    }
    focus .fsBox.path.path}
  bind .fsBox.file.file <Down> {
    global tkVersion
    if {$tkVersion >= 3.0} {
      .fsBox.pattern.pattern icursor end
    } {
      .fsBox.pattern.pattern cursor end
    }
    focus .fsBox.pattern.pattern}

  bind .fsBox.pattern.pattern <Return> {
    global fsBox
    FSBoxFSShow [.fsBox.path.path get]  [.fsBox.pattern.pattern get] $fsBox(all)}
  bind .fsBox.pattern.pattern <Up> {
    global tkVersion
    if {$tkVersion >= 3.0} {
      .fsBox.file.file icursor end
    } {
      .fsBox.file.file cursor end
    }
    focus .fsBox.file.file}
  catch "bind .fsBox.pattern.pattern <Down> {}"

  # packing
  pack append .fsBox.files  .fsBox.files.vscroll "$fsBox(scrollSide) filly"  .fsBox.files.hscroll {bottom fillx}  .fsBox.files.files {left fill expand}
  pack append .fsBox.file  .fsBox.file.labelfile {left}  .fsBox.file.file {left fill expand}
  pack append .fsBox.frame1  .fsBox.frame1.ok {left fill expand}  .fsBox.frame1.rescan {left fill expand}  .fsBox.frame1.cancel {left fill expand}
  pack append .fsBox.path.paths  .fsBox.path.paths.paths {left}
  pack append .fsBox.pattern.patterns  .fsBox.pattern.patterns.patterns {left}
  pack append .fsBox.path  .fsBox.path.paths {left}  .fsBox.path.path {left fill expand}
  pack append .fsBox.pattern  .fsBox.pattern.patterns {left}  .fsBox.pattern.all {right fill}  .fsBox.pattern.pattern {left fill expand}
  if {$fsBox(showPixmap)} {
    pack append .fsBox.frame2  .fsBox.frame2.scrollbar1 {left filly}  .fsBox.frame2.canvas2 {top expand fill}  .fsBox.frame2.scrollbar3 {top fillx} 

    pack append .fsBox  .fsBox.message1 {top fill}  .fsBox.frame1 {bottom fill}  .fsBox.pattern {bottom fill}  .fsBox.file {bottom fill}  .fsBox.path {bottom fill}  .fsBox.frame2 {right fill}  .fsBox.files {left fill expand}
  } {
    pack append .fsBox  .fsBox.message1 {top fill}  .fsBox.frame1 {bottom fill}  .fsBox.pattern {bottom fill}  .fsBox.file {bottom fill}  .fsBox.path {bottom fill}  .fsBox.files {left fill expand}
  }

  if {"$fsBoxActionOk" == "" && "$fsBoxActionCancel" == ""} {
    # wait for the box to be destroyed
    update idletask
    grab .fsBox
    tkwait window .fsBox

    if {"[string trim $fsBox(path)]" != "" ||
        "[string trim $fsBox(name)]" != ""} {
      if {"[string trimleft [string trim $fsBox(name)] /]" == ""} {
        return [string trimright [string trim $fsBox(path)] /]
      } {
        return [string trimright [string trim $fsBox(path)] /]/[string trimleft [string trim $fsBox(name)] /]
      }
    }
  }
}


# Procedure: FSBoxBindSelectOne
proc FSBoxBindSelectOne { fsBoxW fsBoxY} {
# xf ignore me 6

  set fsBoxNearest [$fsBoxW nearest $fsBoxY]
  if {$fsBoxNearest >= 0} {
    $fsBoxW select from $fsBoxNearest
    $fsBoxW select to $fsBoxNearest
  }
}


# Procedure: FSBoxFSFileSelect
proc FSBoxFSFileSelect { fsBoxW fsBoxShowPixmap fsBoxY} {
# xf ignore me 6
  global fsBox

  FSBoxBindSelectOne $fsBoxW $fsBoxY
  set fsBoxNearest [$fsBoxW nearest $fsBoxY]
  if {$fsBoxNearest >= 0} {
    set fsBoxTmpEntry [$fsBoxW get $fsBoxNearest]
    if {"[string index $fsBoxTmpEntry  [expr [string length $fsBoxTmpEntry]-1]]" == "/" ||
        "[string index $fsBoxTmpEntry  [expr [string length $fsBoxTmpEntry]-1]]" == "@"} {
      set fsBoxFileName [string range $fsBoxTmpEntry 0  [expr [string length $fsBoxTmpEntry]-2]]
      if {![IsADir [string trimright $fsBox(internalPath)/$fsBoxFileName @]] &&
          ![IsASymlink [string trimright $fsBox(internalPath)/$fsBoxFileName @]]} {
        set fsBoxFileName $fsBoxTmpEntry
      }
    } {
      if {"[string index $fsBoxTmpEntry  [expr [string length $fsBoxTmpEntry]-1]]" == "*"} {
        set fsBoxFileName [string range $fsBoxTmpEntry 0  [expr [string length $fsBoxTmpEntry]-2]]
        if {![file executable $fsBox(internalPath)/$fsBoxFileName]} {
          set fsBoxFileName $fsBoxTmpEntry
        }
      } {
        set fsBoxFileName $fsBoxTmpEntry
      }
    }
    if {![IsADir [string trimright $fsBox(internalPath)/$fsBoxFileName @]]} {
      set fsBox(name) $fsBoxFileName
      .fsBox.file.file delete 0 end
      .fsBox.file.file insert 0 $fsBox(name)
      if {$fsBoxShowPixmap} {
        catch ".fsBox.frame2.canvas2 itemconfigure currentBitmap -bitmap \"@$fsBox(internalPath)/$fsBox(name)\""
      }
    }
  }
}


# Procedure: FSBoxFSFileSelectDouble
proc FSBoxFSFileSelectDouble { fsBoxW fsBoxShowPixmap fsBoxAction fsBoxY} {
# xf ignore me 6
  global fsBox

  FSBoxBindSelectOne $fsBoxW $fsBoxY
  set fsBoxNearest [$fsBoxW nearest $fsBoxY]
  if {$fsBoxNearest >= 0} {
    set fsBoxTmpEntry [$fsBoxW get $fsBoxNearest]
    if {"$fsBoxTmpEntry" == "../"} {
      set fsBoxTmpEntry [string trimright [string trim $fsBox(internalPath)] "@/"]
      if {"$fsBoxTmpEntry" == ""} {
        return
      }
      FSBoxFSShow [file dirname $fsBoxTmpEntry]  [.fsBox.pattern.pattern get] $fsBox(all)
      .fsBox.path.path delete 0 end
      .fsBox.path.path insert 0 $fsBox(internalPath)
    } {
      if {"[string index $fsBoxTmpEntry  [expr [string length $fsBoxTmpEntry]-1]]" == "/" ||
          "[string index $fsBoxTmpEntry  [expr [string length $fsBoxTmpEntry]-1]]" == "@"} {
        set fsBoxFileName [string range $fsBoxTmpEntry 0  [expr [string length $fsBoxTmpEntry]-2]]
        if {![IsADir [string trimright $fsBox(internalPath)/$fsBoxFileName @]] &&
            ![IsASymlink [string trimright $fsBox(internalPath)/$fsBoxFileName @]]} {
          set fsBoxFileName $fsBoxTmpEntry
        }
      } {
        if {"[string index $fsBoxTmpEntry  [expr [string length $fsBoxTmpEntry]-1]]" == "*"} {
          set fsBoxFileName [string range $fsBoxTmpEntry 0  [expr [string length $fsBoxTmpEntry]-2]]
          if {![file executable $fsBox(internalPath)/$fsBoxFileName]} {
            set fsBoxFileName $fsBoxTmpEntry
          }
        } {
          set fsBoxFileName $fsBoxTmpEntry
        }
      }
      if {[IsADir [string trimright $fsBox(internalPath)/$fsBoxFileName @]]} {
        set fsBox(internalPath) "[string trimright $fsBox(internalPath) {/@}]/$fsBoxFileName"
        FSBoxFSShow $fsBox(internalPath)  [.fsBox.pattern.pattern get] $fsBox(all)
        .fsBox.path.path delete 0 end
        .fsBox.path.path insert 0 $fsBox(internalPath)
      } {
        set fsBox(name) $fsBoxFileName
        if {$fsBoxShowPixmap} {
          set fsBox(path) @$fsBox(internalPath)
        } {
          set fsBox(path) $fsBox(internalPath)
        }
        if {"$fsBoxAction" != ""} {
          eval "global fsBox; $fsBoxAction"
        }
        if {"[info commands XFDestroy]" != ""} {
          catch {XFDestroy .fsBox}
        } {
          catch {destroy .fsBox}
        }
      }
    }
  }
}


# Procedure: FSBoxFSInsertPath
proc FSBoxFSInsertPath {} {
# xf ignore me 6
  global fsBox

  set fsBoxLast [.fsBox.path.paths.paths.menu index last]
  set fsBoxNewEntry [string trimright [.fsBox.path.path get] "/@"]
  for {set fsBoxCounter 0} {$fsBoxCounter <= $fsBoxLast} {incr fsBoxCounter 1} {
    if {"$fsBoxNewEntry" ==  "[lindex [.fsBox.path.paths.paths.menu entryconfigure  $fsBoxCounter -label] 4]"} {
      return
    }
  }
  if {$fsBoxLast < 9} {
    .fsBox.path.paths.paths.menu add command  -label "$fsBoxNewEntry"  -command "
        global fsBox
        FSBoxFSShow $fsBoxNewEntry  \[.fsBox.pattern.pattern get\] \$fsBox(all)
        .fsBox.path.path delete 0 end
        .fsBox.path.path insert 0 $fsBoxNewEntry"
  } {
    for {set fsBoxCounter 0} {$fsBoxCounter < $fsBoxLast} {incr fsBoxCounter 1} {
      .fsBox.path.paths.paths.menu entryconfigure  $fsBoxCounter -label  [lindex [.fsBox.path.paths.paths.menu entryconfigure  [expr $fsBoxCounter+1] -label] 4]
      .fsBox.path.paths.paths.menu entryconfigure $fsBoxCounter  -command "
          global fsBox
          FSBoxFSShow [lindex [.fsBox.path.paths.paths.menu entryconfigure  [expr $fsBoxCounter+1] -label] 4]  \[.fsBox.pattern.pattern get\] \$fsBox(all)
          .fsBox.path.path delete 0 end
          .fsBox.path.path insert 0 [lindex  [.fsBox.path.paths.paths.menu entryconfigure  [expr $fsBoxCounter+1] -label] 4]"
    }
    .fsBox.path.paths.paths.menu entryconfigure $fsBoxLast  -label "$fsBoxNewEntry"
    .fsBox.path.paths.paths.menu entryconfigure $fsBoxCounter  -command "
        global fsBox
        FSBoxFSShow \[.fsBox.path.path get\]  \[.fsBox.pattern.pattern get\] \$fsBox(all)
        .fsBox.path.path delete 0 end
        .fsBox.path.path insert 0 $fsBoxNewEntry"
  }
}


# Procedure: FSBoxFSNameComplete
proc FSBoxFSNameComplete { fsBoxType} {
# xf ignore me 6
  global tkVersion
  global fsBox

  set fsBoxNewFile ""
  if {"$fsBoxType" == "path"} {
    set fsBoxDirName [file dirname [.fsBox.path.path get]]
    set fsBoxFileName [file tail [.fsBox.path.path get]]
  } {
    set fsBoxDirName [file dirname [.fsBox.path.path get]/]
    set fsBoxFileName [file tail [.fsBox.file.file get]]
  }

  set fsBoxNewFile ""
  if {[IsADir [string trimright $fsBoxDirName @]]} {
    catch "glob -nocomplain $fsBoxDirName/${fsBoxFileName}*" fsBoxResult
    foreach fsBoxCounter $fsBoxResult {
      if {"$fsBoxNewFile" == ""} {
        set fsBoxNewFile [file tail $fsBoxCounter]
      } {
        if {"[string index [file tail $fsBoxCounter] 0]" !=
            "[string index $fsBoxNewFile 0]"} {
          set fsBoxNewFile ""
          break
        }
        set fsBoxCounter1 0
        set fsBoxTmpFile1 $fsBoxNewFile
        set fsBoxTmpFile2 [file tail $fsBoxCounter]
        set fsBoxLength1 [string length $fsBoxTmpFile1]
        set fsBoxLength2 [string length $fsBoxTmpFile2]
        set fsBoxNewFile ""
        if {$fsBoxLength1 > $fsBoxLength2} {
          set fsBoxLength1 $fsBoxLength2
        }
        while {$fsBoxCounter1 < $fsBoxLength1} {
          if {"[string index $fsBoxTmpFile1 $fsBoxCounter1]" ==  "[string index $fsBoxTmpFile2 $fsBoxCounter1]"} {
            append fsBoxNewFile [string index $fsBoxTmpFile1 $fsBoxCounter1]
          } {
            break
          }
          incr fsBoxCounter1 1
        }
      }
    }
  }
  if {"$fsBoxNewFile" != ""} {
    if {[IsADir [string trimright $fsBoxDirName/$fsBoxNewFile @]] ||
        ![IsAFile [string trimright $fsBoxDirName/$fsBoxNewFile @]]} {
      if {[IsADir [string trimright $fsBoxDirName/$fsBoxNewFile @]]} {
        if {"$fsBoxDirName" == "/"} {
          .fsBox.path.path delete 0 end
          .fsBox.path.path insert 0 "/[string trimright [string trim $fsBoxNewFile /] @]/"
        } {
          .fsBox.path.path delete 0 end
          .fsBox.path.path insert 0 "[string trimright $fsBoxDirName /]/[string trimright [string trim $fsBoxNewFile /] @]/"
        }
        FSBoxFSShow [.fsBox.path.path get]  [.fsBox.pattern.pattern get] $fsBox(all)
        FSBoxFSInsertPath
      } {
        .fsBox.path.path delete 0 end
        .fsBox.path.path insert 0 "[string trimright $fsBoxDirName /]/[string trimright [string trim $fsBoxNewFile /] @]"
      }
    } {
      .fsBox.path.path delete 0 end
      .fsBox.path.path insert 0 "[string trimright $fsBoxDirName {@/}]/"
      .fsBox.file.file delete 0 end
      .fsBox.file.file insert 0 $fsBoxNewFile
      if {$tkVersion >= 3.0} {
        .fsBox.file.file icursor end
      } {
        .fsBox.file.file cursor end
      }
      focus .fsBox.file.file
    }
  }
}


# Procedure: FSBoxFSShow
proc FSBoxFSShow { fsBoxPath fsBoxPattern fsBoxAll} {
# xf ignore me 6
  global fsBox

  set tmpButtonOpt ""
  if {"$fsBox(activeBackground)" != ""} {
    append tmpButtonOpt "-activebackground \"$fsBox(activeBackground)\" "
  }
  if {"$fsBox(activeForeground)" != ""} {
    append tmpButtonOpt "-activeforeground \"$fsBox(activeForeground)\" "
  }
  if {"$fsBox(background)" != ""} {
    append tmpButtonOpt "-background \"$fsBox(background)\" "
  }
  if {"$fsBox(font)" != ""} {
    append tmpButtonOpt "-font \"$fsBox(font)\" "
  }
  if {"$fsBox(foreground)" != ""} {
    append tmpButtonOpt "-foreground \"$fsBox(foreground)\" "
  }

  set fsBox(pattern) $fsBoxPattern
  if {[file exists $fsBoxPath] && [file readable $fsBoxPath] &&
      [IsADir $fsBoxPath]} {
    set fsBox(internalPath) $fsBoxPath
  } {
    if {[file exists $fsBoxPath] && [file readable $fsBoxPath] &&
        [IsAFile $fsBoxPath]} {
      set fsBox(internalPath) [file dirname $fsBoxPath]
      .fsBox.file.file delete 0 end
      .fsBox.file.file insert 0 [file tail $fsBoxPath]
      set fsBoxPath $fsBox(internalPath)
    } {
      while {"$fsBoxPath" != "" && "$fsBoxPath" != "/" &&
             ![file isdirectory $fsBoxPath]} {
        set fsBox(internalPath) [file dirname $fsBoxPath]
         set fsBoxPath $fsBox(internalPath)
      }
    }
  }
  if {"$fsBoxPath" == ""} {
    set fsBoxPath "/"
    set fsBox(internalPath) "/"
  }
  .fsBox.path.path delete 0 end
  .fsBox.path.path insert 0 $fsBox(internalPath)

  if {[.fsBox.files.files size] > 0} {
    .fsBox.files.files delete 0 end
  }
  if {$fsBoxAll} {
    if {[catch "exec ls -F -a $fsBoxPath" fsBoxResult]} {
      puts stderr "$fsBoxResult"
    }
  } {
    if {[catch "exec ls -F $fsBoxPath" fsBoxResult]} {
      puts stderr "$fsBoxResult"
    }
  }
  set fsBoxElementList [lsort $fsBoxResult]

  foreach fsBoxCounter [winfo children .fsBox.pattern.patterns.patterns] {
    if {[string length [info commands XFDestroy]] > 0} {
      catch {XFDestroy $fsBoxCounter}
    } {
      catch {destroy $fsBoxCounter}
    }
  }
  menu .fsBox.pattern.patterns.patterns.menu
  catch ".fsBox.pattern.patterns.patterns.menu config $tmpButtonOpt"

  if {$fsBox(extensions)} {
    .fsBox.pattern.patterns.patterns.menu add command  -label "*"  -command {
        global fsBox
        set fsBox(pattern) "*"
        .fsBox.pattern.pattern delete 0 end
        .fsBox.pattern.pattern insert 0 $fsBox(pattern)
        FSBoxFSShow [.fsBox.path.path get] $fsBox(pattern)  $fsBox(all)}
  }

  if {"$fsBoxPath" != "/"} {
    .fsBox.files.files insert end "../"
  }
  foreach fsBoxCounter $fsBoxElementList {
    if {[string match $fsBoxPattern $fsBoxCounter] ||
        [IsADir [string trimright $fsBoxPath/$fsBoxCounter "/@"]]} {
      if {"$fsBoxCounter" != "../" &&
          "$fsBoxCounter" != "./"} {
        .fsBox.files.files insert end $fsBoxCounter
      }
    }

    if {$fsBox(extensions)} {
      catch "file rootname $fsBoxCounter" fsBoxRootName
      catch "file extension $fsBoxCounter" fsBoxExtension
      set fsBoxExtension [string trimright $fsBoxExtension "/*@"]
      if {"$fsBoxExtension" != "" && "$fsBoxRootName" != ""} {
        set fsBoxInsert 1
        set fsBoxLast [.fsBox.pattern.patterns.patterns.menu index last]
        for {set fsBoxCounter1 0} {$fsBoxCounter1 <= $fsBoxLast} {incr fsBoxCounter1 1} {
          if {"*$fsBoxExtension" ==  "[lindex [.fsBox.pattern.patterns.patterns.menu entryconfigure  $fsBoxCounter1 -label] 4]"} {
            set fsBoxInsert 0
          }
        }
	if {$fsBoxInsert} {
          .fsBox.pattern.patterns.patterns.menu add command  -label "*$fsBoxExtension"  -command "
              global fsBox
              set fsBox(pattern) \"*$fsBoxExtension\"
              .fsBox.pattern.pattern delete 0 end
              .fsBox.pattern.pattern insert 0 \$fsBox(pattern)
              FSBoxFSShow \[.fsBox.path.path get\] \$fsBox(pattern)  \$fsBox(all)"
        }
      }
    }
  }
  if {$fsBox(extensions)} {
    .fsBox.pattern.patterns.patterns.menu add separator
  }
  if {$fsBox(extensions) || 
      "[.fsBox.pattern.patterns.patterns.menu index last]" == "none"} {
    .fsBox.pattern.patterns.patterns.menu add checkbutton  -label "Scan extensions"  -variable "fsBox(extensions)"  -command {
        global fsBox
        FSBoxFSShow [.fsBox.path.path get]  [.fsBox.pattern.pattern get] $fsBox(all)}
  }
}


# Procedure: IsADir
proc IsADir { pathName} {
# xf ignore me 5
##########
# Procedure: IsADir
# Description: check if name is a directory (including symbolic links)
# Arguments: pathName - the path to check
# Returns: 1 if its a directory, otherwise 0
# Sideeffects: none
##########

  if {[file isdirectory $pathName]} {
    return 1
  } {
    catch "file type $pathName" fileType
    if {"$fileType" == "link"} {
      if {[catch "file readlink $pathName" linkName]} {
        return 0
      }
      catch "file type $linkName" fileType
      while {"$fileType" == "link"} {
        if {[catch "file readlink $linkName" linkName]} {
          return 0
        }
        catch "file type $linkName" fileType
      }
      return [file isdirectory $linkName]
    }
  }
  return 0
}


# Procedure: IsAFile
proc IsAFile { fileName} {
# xf ignore me 5
##########
# Procedure: IsAFile
# Description: check if filename is a file (including symbolic links)
# Arguments: fileName - the filename to check
# Returns: 1 if its a file, otherwise 0
# Sideeffects: none
##########

  if {[file isfile $fileName]} {
    return 1
  } {
    catch "file type $fileName" fileType
    if {"$fileType" == "link"} {
      if {[catch "file readlink $fileName" linkName]} {
        return 0
      }
      catch "file type $linkName" fileType
      while {"$fileType" == "link"} {
        if {[catch "file readlink $linkName" linkName]} {
          return 0
        }
        catch "file type $linkName" fileType
      }
      return [file isfile $linkName]
    }
  }
  return 0
}


# Procedure: IsASymlink
proc IsASymlink { fileName} {
# xf ignore me 5
##########
# Procedure: IsASymlink
# Description: check if filename is a symbolic link
# Arguments: fileName - the path/filename to check
# Returns: none
# Sideeffects: none
##########

  catch "file type $fileName" fileType
  if {"$fileType" == "link"} {
    return 1
  }
  return 0
}

proc get_provider_offset { c } {
    global pOffsetX pOffsetY
    regsub -all " " $c "_" city
    if {! [info exists pOffsetX($city)] } {
	return [ list 0 0]
    } else {
	return [ list $pOffsetX($city) $pOffsetY($city) ]
    }
}

proc get_text_offset { c } {
    global cityX cityY zoomFactor
    regsub -all " " $c "_" city
    set l [.f.canvas coords $city]
    set xoff [ expr  [lindex $l 0] / $zoomFactor - $cityX($city)]
    set yoff [ expr  [lindex $l 1] / $zoomFactor - $cityY($city)]
    return [ list $xoff $yoff]
}

proc move_provider {ignore } {
    global textXoffset textYoffset
    global pOffsetX pOffsetY
    regsub -all " " [.city.workarea.city.entry get] "_" city
    set pOffsetX($city) $textXoffset 
    set pOffsetY($city) $textYoffset 
    arrange_providers $city
}

proc scale_text { size } {
    global textSize zoomFactor
    regsub -all " " [.city.workarea.city.entry get] "_" city 
    set textSize($city) $size
    .f.canvas itemconfigure $city \
	-font [make_font [expr round($size * $zoomFactor) ]]
}


proc move_text {ignore } {
    global textXoffset textYoffset cityX cityY zoomFactor
    global moveWhat drawArrow
    if { $moveWhat == "text" } {
	regsub -all " " [.city.workarea.city.entry get]  "_" city
	set tx [ expr ($cityX($city) + $textXoffset) * $zoomFactor]
	set ty [ expr ($cityY($city) + $textYoffset) * $zoomFactor]
	.f.canvas coords $city $tx $ty

    } else {
	move_provider 0
    }
    if { $drawArrow } {
	draw_arrow
    }
}

proc terminate_city { } {
    global cityState
    if {$cityState == "new"} {
	add_new_city [.city.workarea.city.entry get] \
	             [.city.workarea.prov.frame11.sel_providers get 0 end] \
	             [.city.workarea.coords.xcoordinate.entry get] \
                     [.city.workarea.coords.ycoordinate.entry get] 
    } else {
	set city [.city.workarea.city.entry get]
	modify_city $city
    }
}
proc make_font { size} {
    return "-adobe-helvetica-bold-r-normal--*-${size}-*-*-p-*-iso8859-1"
}

proc get_font_size { font } {
# Foundry   Name   Weight Slant Mod.     Pixel   Pts      Xpix    Ypix     Res.  Spac.
    regexp -- \
	{-([^-]+-[^-]+-[^-]+-.-[^-]+)--([0-9*]+)-([0-9*]+)-([0-9*]+)-([0-9*]+)-([0-9*]+)-(.)-([0-9*]+)-(.*$)} \
	$font ignore longname pixels points xpix ypix res \
	space charset
    return $points
}

proc create_provider { city provider } {
    global cityX cityY provColor rectSide zoomFactor provList
    if { [lsearch $provList $provider] < 0  } {
	puts "Provider '$provider' existiert nicht (Stadt: '$city')"
	return
    }
    if { [.f.canvas gettags ${city}_${provider}] != "" } {
#	puts "Provider '$provider' schon bei '$city' eingetragen!"
	return
    }
    set col $provColor($provider)
    set x1 [expr $cityX($city) * $zoomFactor]
    set y1 [expr $cityY($city) * $zoomFactor]
    set x2 [expr $x1 + $rectSide * $zoomFactor]
    set y2 [expr $y1 + $rectSide * $zoomFactor]
    .f.canvas create rectangle $x1 $y1 $x2 $y2 -fill $col \
	   -tags "${city}_${provider}" -outline $col
}

proc arrange_providers { city} {
    global cityX cityY rectSide rectSpace zoomFactor
    global cityProviders pOffsetX pOffsetY

    if { ![info exists cityProviders($city)] } {
	return
    }
    set c .f.canvas
    set provlist $cityProviders($city)
#    puts "Arranging providers for '$city': $provlist"
    set count [llength $provlist]
    if { $count == 0 } { return }

    set base  [expr round(sqrt($count))]
    set delta [expr $rectSide + $rectSpace]
    set tag ${city}_moved
#
#   define lower left corner of first rectangle:
#   left of city center, bottom on city center
#
    if { [info exists pOffsetX($city) ] } {
	set tx $pOffsetX($city)
	set ty $pOffsetY($city)
    } else {
	set tx 0.0
	set ty 0.0
    }
    set xs \
	[expr ($cityX($city) + $tx - $base * $delta / 2.0) * \
	     $zoomFactor]
    set ys [expr ($cityY($city) + $ty ) * $zoomFactor]
    set x0 $xs
    set delta [expr $delta * $zoomFactor]
    set i 0
    foreach p $provlist {
	incr i
	set tag ${city}_${p}
        set clist [$c coords $tag]
	set x [lindex $clist 0]
	set y [lindex $clist 1]
	set xt [expr $xs - $x]
	set yt [expr $ys - $y]
	.f.canvas move $tag $xt $yt
	if { $i ==  $base  } {
#	    puts "$i == $base"
	    set xs $x0
	    set ys [expr $ys - $delta]
	    set i 0
	} else {
	    set xs [ expr $xs + $delta ]
	}
    }
}

proc modify_city {citystring} { 
    global fontSize cityX cityY zoomFactor
    global pOffsetX pOffsetY textSize
    set c .f.canvas
    regsub -all " " $citystring "_" city
#
#   Get the list of existing and not-existing ('deleted') providers for
#   this city. Remove all rectangles for the deleted providers
#
    set exists [lsort [.city.workarea.prov.frame11.sel_providers get 0 end]]
    set deleted [lsort [.city.workarea.prov.frame.all_providers get 0 end]]
    foreach i $deleted {
	set tag ${city}_${i}
	if { [$c gettags $tag ] != "" } {
	      $c delete $tag
	}
    }
    foreach i $exists {
	set tag ${city}_${i}
	if { [$c gettags $tag ] == "" } {
	    create_provider  $city $i
	}
    }
    set cityProviders($city) $exists
    arrange_providers $city
#
#   Update the labelling by setting the font size and the tags
#
    set textSize($city) $fontSize
    $c itemconfigure $city \
	-font [make_font [expr round($fontSize * $zoomFactor)]] \
	-tags [concat $city $exists] 
    $c raise $city 
}

proc draw_arrow {} {
    global zoomFactor drawArrow cityX cityY pOffsetX pOffsetY
    #
    #  Draw line from city name or provider(s) to
    #  exact city position. End point is always the
    #  city position, end point is either the city name
    #  or the provider position, what ever is further awaz
    #  from the city's position
    #
    regsub -all " " [.city.workarea.city.entry get] "_" city

    set tag ${city}_line
    set c .f.canvas
    if { ! $drawArrow } {
	$c delete $tag
        return
    }
    if  { ! [info exists pOffsetX($city) ] } {
	set pOffsetX($city) 0
	set pOffsetY($city) 0
    }
    set xp [expr ($pOffsetX($city) + $cityX($city)) * $zoomFactor ]
    set yp [expr ($pOffsetY($city) + $cityY($city)) * $zoomFactor ]
    set lc [ $c coords $city ]
    set xt [ lindex $lc 0 ]
    set yt [ lindex $lc 1 ]
    set xs [expr $cityX($city) * $zoomFactor ]
    set ys [expr $cityY($city) * $zoomFactor ]
    set dx1 [ expr $xp - $xs]
    set dx2 [ expr $xt - $xs]
    set dy1 [ expr $yp - $ys]
    set dy2 [ expr $yt - $ys]
    set d1 [ expr sqrt($dx1 * $dx1 + $dy1 * $dy1) ]
    set d2 [ expr sqrt($dx2 * $dx2 + $dy2 * $dy2) ]
    if { $d1 > $d2 } {
	set xe $xp
	set ye $yp
	set l $d1
    } else {
	set xe $xt
	set ye $yt
	set l $d2
    }
    if { [$c gettags $tag] == "" } {
	set al [expr 2 * $zoomFactor ]
	set aw [expr 2 * $zoomFactor ]
	set ah [expr 1.5 * $zoomFactor ]
	$c create line \
	    $xs $ys $xe $ye -arrow first\
	    -tags $tag -fill black -width 0.2 -arrowshape [list $al $aw $ah ]
    } else {
	$c coords $tag \
	    $xs $ys $xe $ye
    }
}

# Procedure: add_new_city
proc add_new_city { city plist xc yc } {
    global provColor cityX cityY zoomFactor fontSize textSize
    set pcount [llength $plist]
    regsub -all " " $city "_" norm_city
    set cityX($norm_city) $xc
    set cityY($norm_city) $yc
    foreach p $plist {
	create_provider $norm_city $p
    }
    set textSize($norm_city) $fontSize 
    .f.canvas create \
	text [expr $xc * $zoomFactor] [expr $yc * $zoomFactor ] \
	-text "$city" -anchor w \
	-tags [concat $norm_city $plist] \
	-font [make_font [expr round($fontSize * $zoomFactor) ]]
    .f.canvas bind $norm_city <1> "show_city $norm_city"
    arrange_providers $norm_city
}


# startup source
proc StartupSrc {args} {
global tkVersion
# set tkVersion 4.0
}



# startup source
StartupSrc

# initialize global variables
proc InitGlobals {} {
  global {fsBox}
  set {fsBox(activeBackground)} {}
  set {fsBox(activeForeground)} {}
  set {fsBox(all)} {0}
  set {fsBox(background)} {}
  set {fsBox(button)} {0}
  set {fsBox(extensions)} {0}
  set {fsBox(font)} {}
  set {fsBox(foreground)} {}
  set {fsBox(internalPath)} {/home/ck}
  set {fsBox(name)} {}
  set {fsBox(path)} {/home/ck}
  set {fsBox(pattern)} {*}
  set {fsBox(scrollActiveForeground)} {}
  set {fsBox(scrollBackground)} {}
  set {fsBox(scrollForeground)} {}
  set {fsBox(scrollSide)} {left}
  set {fsBox(showPixmap)} {0}
}

# initialize global variables
InitGlobals

# display/remove toplevel windows.
ShowWindow.

set zoomFactor 1.0

global xfShowWindow.city .f.canvas
set xfShowWindow.city 1
#ShowWindow.city

# read provider file

set pfile [open "providers.lst" r]
while { [gets $pfile line] >= 0 } {
    regexp {([0-9A-Za-z][^ 	]*)[ 	]*(.*)$} $line ignore name c
#    puts "$name -> $c"
    lappend provList $name
    set provColor($name) $c
}
set provList [lsort $provList]
close $pfile 

if { [file exists "karte.cnv"] } {
    set content ""
    set cfile [open "karte.cnv" r]
    while { [gets $cfile line] >= 0 } {
	if { [regexp "^set" "$line"] } {
#	    puts "Evaluating [split $line " "]"
	    eval $line
	} else {
	    undump_canvas .f.canvas $line
	}
    }
    close $cfile
}


if { [file exists "city.coords" ] } {
    set cfile [open "city.coords" r ] 
    while { [gets $cfile line ] >= 0 } {
	regexp {([^ 	]*)[ 	]*([0-9.-]*)[ 	]*([0-9.-]*)$} \
	    $line ignore name x y
	lappend cities $name
	set found [.f.canvas find withtag $name]
	if { [llength $found] == 0 } {
	    regsub -all "_" $name " " norm_name
	    add_new_city "$norm_name" {} $x $y
	}
    }
    close $cfile
}


if { [file exists "city.providers" ] } {
    set cfile [open "city.providers" r ] 
    while { [gets $cfile line ] >= 0 } {
	regexp {([^ 	]*)[ 	]*(.*)$} \
	    $line ignore name p
	if { [lsearch $cities $name] < 0 } {
	    puts "Stadt '$name' nicht gefunden!"
	} else {
	    regsub -all {[ 	]+} $p " " providers
	    set plist [split $providers " "]
	    foreach p $plist {
		if { [string length $p] && [lsearch $provList $p] >= 0} {
		    create_provider $name $p
		    .f.canvas addtag $p withtag $name
		    lappend cityProviders($name) $p
		}
	    }
	    arrange_providers $name
	    .f.canvas raise $name
	}
    }
    close $cfile
}

#
# build missing textSize entries
#
foreach i $cities {
    if { ![info exists textSize($i)]} {
	set textSize($i) [get_font_size [.f.canvas itemcget $i -font]]
    }
}
# load default bindings.
if {[info exists env(XF_BIND_FILE)] &&
    "[info procs XFShowHelp]" == ""} {
  source $env(XF_BIND_FILE)
}

# parse and apply application defaults.

# eof
#

